@@ -73,7 +73,7 @@ fluid_cmd_t fluid_commands[] = {
   { "inst", "general", (fluid_cmd_func_t) fluid_handle_inst, NULL,
     "inst font                  Print out the available instruments for the font" },
   { "channels", "general", (fluid_cmd_func_t) fluid_handle_channels, NULL,
-    "channels                   Print out preset of all channels" },
+    "channels [-verbose]        Print out preset of all channels" },
   { "interp", "general", (fluid_cmd_func_t) fluid_handle_interp, NULL,
     "interp num                 Choose interpolation method for all channels" },
   { "interpc", "general", (fluid_cmd_func_t) fluid_handle_interpc, NULL,
@@ -267,6 +267,7 @@ int fluid_shell_run(fluid_shell_t* shell)
   char* prompt = "";
   int cont = 1;
   int errors = 0;
+  int n;
 
   if (shell->settings) {
     fluid_settings_getstr(shell->settings, "shell.prompt", &prompt);
@@ -275,7 +276,9 @@ int fluid_shell_run(fluid_shell_t* shell)
   /* handle user input */
   while (cont) {
     
-    if (fluid_istream_readline(shell->in, prompt, workline, FLUID_WORKLINELENGTH) != 0) {
+    n = fluid_istream_readline(shell->in, prompt, workline, FLUID_WORKLINELENGTH);
+
+    if (n < 0) {
       break;
     }
 
@@ -300,6 +303,10 @@ int fluid_shell_run(fluid_shell_t* shell)
       cont = 0;
       break;
     }
+
+    if (n == 0) {
+       break;
+    }
   }
 
   return errors;
@@ -321,14 +328,13 @@ fluid_source(fluid_cmd_handler_t* handler, char* filename)
   fluid_shell_t shell;
 
 #ifdef WIN32 
-  file = _open(filename, _O_WRONLY);
+  file = _open(filename, _O_RDONLY);
 #else
-  file = open(filename, O_WRONLY);
+  file = open(filename, O_RDONLY);
 #endif
   if (file < 0) {
     return file;
   }
-
   fluid_shell_init(&shell, NULL, handler, file, fluid_get_stdout());
   return fluid_shell_run(&shell);
 }
@@ -480,15 +486,19 @@ fluid_handle_channels(fluid_synth_t* synth, int ac, char** av, fluid_ostream_t o
 {
   int i;
   fluid_preset_t* preset;
-
+  int verbose = 0;
+  
+  if (ac > 0 && strcmp( av[0], "-verbose") == 0) verbose = 1;
+  
   for (i = 0; i < fluid_synth_count_midi_channels(synth); i++) {
     preset = fluid_synth_get_channel_preset(synth, i);
-    if (preset != NULL) {
-      fluid_ostream_printf(out, "chan %d, %s\n", 
-			  i, fluid_preset_get_name(preset));
-    } else {
-      fluid_ostream_printf(out, "chan %d, no preset\n", i);
-    }
+    if (preset == NULL) fluid_ostream_printf(out, "chan %d, no preset\n", i);
+    else if (!verbose) fluid_ostream_printf(out, "chan %d, %s\n", i, fluid_preset_get_name(preset));
+    else fluid_ostream_printf(out, "chan %d, sfont %d, bank %d, preset %d, %s\n", i,
+                              fluid_sfont_get_id( preset->sfont),
+                              fluid_preset_get_banknum(preset),
+                              fluid_preset_get_num(preset),
+                              fluid_preset_get_name(preset));
   }
   return 0;
 }
@@ -1180,7 +1190,7 @@ fluid_handle_get(fluid_synth_t* synth, int ac, char** av, fluid_ostream_t out)
   case FLUID_NUM_TYPE: {
     double value;
     fluid_synth_getnum(synth, av[0], &value);
-    fluid_ostream_printf(out, "%f", value);
+    fluid_ostream_printf(out, "%.3f", value);
     break;
   }
 
@@ -1237,7 +1247,7 @@ static void fluid_handle_settings_iter2(void* data, char* name, int type)
   case FLUID_NUM_TYPE: {
     double value;
     fluid_synth_getnum(d->synth, name, &value);
-    fluid_ostream_printf(d->out, "%f\n", value);
+    fluid_ostream_printf(d->out, "%.3f\n", value);
     break;
   }
     
@@ -1311,7 +1321,7 @@ fluid_handle_info(fluid_synth_t* synth, int ac, char** av, fluid_ostream_t out)
     fluid_settings_getnum(settings, av[0], &value);
     fluid_ostream_printf(out, "%s:\n", av[0]);
     fluid_ostream_printf(out, "Type:          number\n");
-    fluid_ostream_printf(out, "Value:         %f\n", value);
+    fluid_ostream_printf(out, "Value:         %.3f\n", value);
     fluid_ostream_printf(out, "Minimum value: %.3f\n", min);
     fluid_ostream_printf(out, "Maximum value: %.3f\n", max);
     fluid_ostream_printf(out, "Default value: %.3f\n", 
