@@ -29,6 +29,29 @@
 
 fluid_sfloader_t* new_fluid_defsfloader(void);
 
+/************************************************************************
+ *
+ * These functions were added after the v1.0 API freeze. They are not
+ * in synth.h. They should be added as soon as a new development
+ * version is started.
+ *
+ ************************************************************************/
+
+int fluid_synth_program_select2(fluid_synth_t* synth, 
+				int chan, 
+				char* sfont_name, 
+				unsigned int bank_num, 
+				unsigned int preset_num);
+
+fluid_sfont_t* fluid_synth_get_sfont_by_name(fluid_synth_t* synth, char *name);
+
+int fluid_synth_get_pitch_wheel_sens(fluid_synth_t* synth, int chan, int* pval);
+
+int fluid_synth_set_gen2(fluid_synth_t* synth, int chan, 
+			 int param, float value, 
+			 int absolute, int normalized);
+
+ 
 /***************************************************************
  *
  *                         GLOBAL
@@ -352,14 +375,13 @@ new_fluid_synth(fluid_settings_t *settings)
 			      (fluid_num_update_t) fluid_synth_update_gain, synth);
 
   /* do some basic sanity checking on the settings */
-  if (synth->midi_channels < 16) {
-    FLUID_LOG(FLUID_WARN, "Requested number of MIDI channels is smaller than 16. " 
-	     "Changing this setting to 16."); 
-    synth->midi_channels = 16;
-  } else if (synth->midi_channels > 1024) {
-    FLUID_LOG(FLUID_WARN, "Requested number of MIDI channels is too big (%d). " 
-	     "Limiting this setting to 1024.", synth->midi_channels); 
-    synth->midi_channels = 1024;
+
+  if (synth->midi_channels % 16 != 0) {
+    int n = synth->midi_channels / 16;
+    synth->midi_channels = (n + 1) * 16;
+    fluid_settings_setint(settings, "synth.midi-channels", synth->midi_channels);
+    FLUID_LOG(FLUID_WARN, "Requested number of MIDI channels is not a multiple of 16. " 
+	     "I'll increase the number of channels to the next multiple."); 
   }
 
   if (synth->audio_channels < 1) {
@@ -672,7 +694,8 @@ delete_fluid_synth(fluid_synth_t* synth)
   FLUID_FREE(synth->LADSPA_FxUnit);
 #endif
   
-  fluid_mutex_destroy(synth->busy);  
+  fluid_mutex_destroy(synth->busy);
+
   FLUID_FREE(synth);
 
   return FLUID_OK;
@@ -697,24 +720,12 @@ int
 fluid_synth_noteon(fluid_synth_t* synth, int chan, int key, int vel) 
 {
   fluid_channel_t* channel;
-  int r;
-  fluid_mutex_lock(synth->busy); /* Don't interfere with the audio thread */
-  fluid_mutex_unlock(synth->busy);
+  int r = FLUID_FAILED;
 
   /* check the ranges of the arguments */
   if ((chan < 0) || (chan >= synth->midi_channels)) {
     FLUID_LOG(FLUID_WARN, "Channel out of range");
-    return FLUID_FAILED;     
-  }
-
-  if ((key < 0) || (key >= 128)) {
-    FLUID_LOG(FLUID_WARN, "Key out of range");
-    return FLUID_FAILED;     
-  }
-
-  if ((vel < 0) || (vel >= 128)) {
-    FLUID_LOG(FLUID_WARN, "Velocity out of range");
-    return FLUID_FAILED;     
+    return FLUID_FAILED;
   }
 
   /* notes with velocity zero go to noteoff  */
@@ -722,8 +733,6 @@ fluid_synth_noteon(fluid_synth_t* synth, int chan, int key, int vel)
     return fluid_synth_noteoff(synth, chan, key);
   }
 
-  synth->noteid++;
-
   channel = synth->channel[chan];
 
   /* make sure this channel has a preset */
@@ -735,12 +744,10 @@ fluid_synth_noteon(fluid_synth_t* synth, int chan, int key, int vel)
 	       (fluid_curtime() - synth->start) / 1000.0f,
 	       0.0f, 0, "channel has no preset");
     }
-    return FLUID_FAILED; 
+    return FLUID_FAILED;
   } 
 
-  r = fluid_preset_noteon(channel->preset, synth, chan, key, vel);
-
-  return r;
+  return fluid_synth_start(synth, synth->noteid++, channel->preset, 0, chan, key, vel);
 }
 
 /*
@@ -752,8 +759,8 @@ fluid_synth_noteoff(fluid_synth_t* synth, int chan, int key)
   int i;
   fluid_voice_t* voice;
   int status = FLUID_FAILED;
-  fluid_mutex_lock(synth->busy); /* Don't interfere with the audio thread */
-  fluid_mutex_unlock(synth->busy);
+/*   fluid_mutex_lock(synth->busy); /\* Don't interfere with the audio thread *\/ */
+/*   fluid_mutex_unlock(synth->busy); */
   
   for (i = 0; i < synth->nvoice; i++) {
     voice = synth->voice[i];
@@ -768,7 +775,7 @@ fluid_synth_noteoff(fluid_synth_t* synth, int chan, int key)
 	}
 	FLUID_LOG(FLUID_INFO, "noteoff\t%d\t%d\t%d\t%05d\t%.3f\t%.3f\t%.3f\t%d", 
 		 voice->chan, voice->key, 0, voice->id, 
-		 (float) (voice->start + voice->ticks) / 44100.0f,
+		 (float) (voice->start_time + voice->ticks) / 44100.0f,
 		 (fluid_curtime() - synth->start) / 1000.0f,
 		 (float) voice->ticks / 44100.0f,
 		 used_voices);
@@ -789,8 +796,9 @@ fluid_synth_damp_voices(fluid_synth_t* synth, int chan)
   int i;
   fluid_voice_t* voice;
 
-  fluid_mutex_lock(synth->busy); /* Don't interfere with the audio thread */
-  fluid_mutex_unlock(synth->busy);
+/*   fluid_mutex_lock(synth->busy); /\* Don't interfere with the audio thread *\/ */
+/*   fluid_mutex_unlock(synth->busy); */
+
   for (i = 0; i < synth->nvoice; i++) {
     voice = synth->voice[i];
     if ((voice->chan == chan) && _SUSTAINED(voice)) {
@@ -808,8 +816,9 @@ fluid_synth_damp_voices(fluid_synth_t* synth, int chan)
 int 
 fluid_synth_cc(fluid_synth_t* synth, int chan, int num, int val)
 {
-  fluid_mutex_lock(synth->busy); /* Don't interfere with the audio thread */
-  fluid_mutex_unlock(synth->busy);
+/*   fluid_mutex_lock(synth->busy); /\* Don't interfere with the audio thread *\/ */
+/*   fluid_mutex_unlock(synth->busy); */
+
   /* check the ranges of the arguments */
   if ((chan < 0) || (chan >= synth->midi_channels)) {
     FLUID_LOG(FLUID_WARN, "Channel out of range");
@@ -935,8 +944,9 @@ fluid_synth_modulate_voices(fluid_synth_t* synth, int chan, int is_cc, int ctrl)
   int i;
   fluid_voice_t* voice;
 
-  fluid_mutex_lock(synth->busy); /* Don't interfere with the audio thread */
-  fluid_mutex_unlock(synth->busy);
+/*   fluid_mutex_lock(synth->busy); /\* Don't interfere with the audio thread *\/ */
+/*   fluid_mutex_unlock(synth->busy); */
+
   for (i = 0; i < synth->nvoice; i++) {
     voice = synth->voice[i];
     if (voice->chan == chan) {
@@ -958,8 +968,9 @@ fluid_synth_modulate_voices_all(fluid_synth_t* synth, int chan)
 {
   int i;
   fluid_voice_t* voice;
-  fluid_mutex_lock(synth->busy); /* Don't interfere with the audio thread */
-  fluid_mutex_unlock(synth->busy);
+
+/*   fluid_mutex_lock(synth->busy); /\* Don't interfere with the audio thread *\/ */
+/*   fluid_mutex_unlock(synth->busy); */
 
   for (i = 0; i < synth->nvoice; i++) {
     voice = synth->voice[i];
@@ -977,8 +988,9 @@ int
 fluid_synth_pitch_bend(fluid_synth_t* synth, int chan, int val)
 {
 
-  fluid_mutex_lock(synth->busy); /* Don't interfere with the audio thread */
-  fluid_mutex_unlock(synth->busy);
+/*   fluid_mutex_lock(synth->busy); /\* Don't interfere with the audio thread *\/ */
+/*   fluid_mutex_unlock(synth->busy); */
+
   /* check the ranges of the arguments */
   if ((chan < 0) || (chan >= synth->midi_channels)) {
     FLUID_LOG(FLUID_WARN, "Channel out of range");
@@ -1041,7 +1053,7 @@ fluid_synth_pitch_wheel_sens(fluid_synth_t* synth, int chan, int val)
  * So its API is not in the synth.h file. It should be added in some later
  * version of fluidsynth. Maybe v2.0 ? -- Antoine Schmitt May 2003
  */
-/*******************
+
 int 
 fluid_synth_get_pitch_wheel_sens(fluid_synth_t* synth, int chan, int* pval)
 {
@@ -1057,7 +1069,6 @@ fluid_synth_get_pitch_wheel_sens(fluid_synth_t* synth, int chan, int* pval)
 
   return FLUID_OK;
 }
-************/
 
 /*
  * fluid_synth_get_preset
@@ -1081,6 +1092,27 @@ fluid_synth_get_preset(fluid_synth_t* synth, unsigned int sfontnum,
   return NULL;
 }
 
+/*
+ * fluid_synth_get_preset2
+ */
+fluid_preset_t*
+fluid_synth_get_preset2(fluid_synth_t* synth, char* sfont_name, 
+			unsigned int banknum, unsigned int prognum)
+{
+  fluid_preset_t* preset = NULL;
+  fluid_sfont_t* sfont = NULL;
+
+  sfont = fluid_synth_get_sfont_by_name(synth, sfont_name);
+  
+  if (sfont != NULL) {
+    preset = fluid_sfont_get_preset(sfont, banknum, prognum);
+    if (preset != NULL) {
+      return preset;
+    }
+  }
+  return NULL;
+}
+
 fluid_preset_t* fluid_synth_find_preset(fluid_synth_t* synth, 
 				      unsigned int banknum, 
 				      unsigned int prognum)
@@ -1117,8 +1149,9 @@ fluid_synth_program_change(fluid_synth_t* synth, int chan, int prognum)
   unsigned int banknum;
   unsigned int sfont_id;
 
-  fluid_mutex_lock(synth->busy); /* Don't interfere with the audio thread */
-  fluid_mutex_unlock(synth->busy);
+/*   fluid_mutex_lock(synth->busy); /\* Don't interfere with the audio thread *\/ */
+/*   fluid_mutex_unlock(synth->busy); */
+
   if ((prognum >= 0) && (prognum < FLUID_NUM_PROGRAMS) &&
       (chan >= 0) && (chan < synth->midi_channels)) {
 
@@ -1238,6 +1271,49 @@ int fluid_synth_program_select(fluid_synth_t* synth,
   return FLUID_OK;
 }
 
+/*
+ * fluid_synth_program_select2
+ */
+int fluid_synth_program_select2(fluid_synth_t* synth, 
+				int chan, 
+				char* sfont_name, 
+				unsigned int bank_num, 
+				unsigned int preset_num)
+{
+  fluid_preset_t* preset = NULL;
+  fluid_channel_t* channel;
+  fluid_sfont_t* sfont = NULL;
+
+  if ((chan < 0) || (chan >= synth->midi_channels)) {
+    FLUID_LOG(FLUID_ERR, "Channel number out of range (chan=%d)", chan);
+    return FLUID_FAILED;
+  }
+  channel = synth->channel[chan];
+
+  sfont = fluid_synth_get_sfont_by_name(synth, sfont_name);
+  if (sfont == NULL) {
+    FLUID_LOG(FLUID_ERR, "Could not find SoundFont %s", sfont_name);
+    return FLUID_FAILED;
+  }
+
+  preset = fluid_sfont_get_preset(sfont, bank_num, preset_num);
+  if (preset == NULL) {
+    FLUID_LOG(FLUID_ERR, 
+	      "There is no preset with bank number %d and preset number %d in SoundFont %s", 
+	      bank_num, preset_num, sfont_name);
+    return FLUID_FAILED;
+  }
+
+  /* inform the channel of the new bank and program number */
+  fluid_channel_set_sfontnum(channel, fluid_sfont_get_id(sfont));
+  fluid_channel_set_banknum(channel, bank_num);
+  fluid_channel_set_prognum(channel, preset_num);
+
+  fluid_channel_set_preset(channel, preset);
+
+  return FLUID_OK;
+}
+
 /*
  * fluid_synth_update_presets
  */
@@ -1342,8 +1418,9 @@ int fluid_synth_set_reverb_preset(fluid_synth_t* synth, int num)
 void fluid_synth_set_reverb(fluid_synth_t* synth, double roomsize, double damping, 
 			   double width, double level)
 {
-  fluid_mutex_lock(synth->busy); /* Don't interfere with the audio thread */
-  fluid_mutex_unlock(synth->busy);
+/*   fluid_mutex_lock(synth->busy); /\* Don't interfere with the audio thread *\/ */
+/*   fluid_mutex_unlock(synth->busy); */
+
   fluid_revmodel_setroomsize(synth->reverb, roomsize);
   fluid_revmodel_setdamp(synth->reverb, damping);
   fluid_revmodel_setwidth(synth->reverb, width);
@@ -1356,8 +1433,9 @@ void fluid_synth_set_reverb(fluid_synth_t* synth, double roomsize, double dampin
 void fluid_synth_set_chorus(fluid_synth_t* synth, int nr, double level, 
 			   double speed, double depth_ms, int type)
 {
-  fluid_mutex_lock(synth->busy); /* Don't interfere with the audio thread */
-  fluid_mutex_unlock(synth->busy);
+/*   fluid_mutex_lock(synth->busy); /\* Don't interfere with the audio thread *\/ */
+/*   fluid_mutex_unlock(synth->busy); */
+
   fluid_chorus_set_nr(synth->chorus, nr);
   fluid_chorus_set_level(synth->chorus, (fluid_real_t)level);
   fluid_chorus_set_speed_Hz(synth->chorus, (fluid_real_t)speed);
@@ -1416,7 +1494,7 @@ void fluid_synth_set_chorus(fluid_synth_t* synth, int nr, double level,
 ***************************************************/
 
 /* 
- *  fluid_synth_write_float
+ *  fluid_synth_nwrite_float
  */
 int 
 fluid_synth_nwrite_float(fluid_synth_t* synth, int len, 
@@ -1454,21 +1532,21 @@ fluid_synth_nwrite_float(fluid_synth_t* synth, int len,
 
   /* Then, run one_block() and copy till we have 'len' samples  */
   while (count < len) {
-      fluid_synth_one_block(synth, 1);
+    fluid_synth_one_block(synth, 1);
 
-      num = (FLUID_BUFSIZE > len - count)? len - count : FLUID_BUFSIZE;
-      bytes = num * sizeof(float);
-      
-      for (i = 0; i < synth->audio_channels; i++) {
-	FLUID_MEMCPY(left[i] + count, left_in[i], bytes);
-	FLUID_MEMCPY(right[i] + count, right_in[i], bytes);
-      }
-      for (i = 0; i < synth->effects_channels; i++) {
-	FLUID_MEMCPY(fx_left[i] + count, fx_left_in[i], bytes);
-	FLUID_MEMCPY(fx_right[i] + count, fx_right_in[i], bytes);
-      }
-
-      count += num;
+    num = (FLUID_BUFSIZE > len - count)? len - count : FLUID_BUFSIZE;
+    bytes = num * sizeof(float);
+    
+    for (i = 0; i < synth->audio_channels; i++) {
+      FLUID_MEMCPY(left[i] + count, left_in[i], bytes);
+      FLUID_MEMCPY(right[i] + count, right_in[i], bytes);
+    }
+    for (i = 0; i < synth->effects_channels; i++) {
+      FLUID_MEMCPY(fx_left[i] + count, fx_left_in[i], bytes);
+      FLUID_MEMCPY(fx_right[i] + count, fx_right_in[i], bytes);
+    }
+    
+    count += num;
   }
 
   synth->cur = num;
@@ -1621,7 +1699,8 @@ fluid_synth_one_block(fluid_synth_t* synth, int do_not_mix_fx_to_out)
   fluid_real_t* chorus_buf;
   int byte_size = FLUID_BUFSIZE * sizeof(fluid_real_t);
   double prof_ref = fluid_profile_ref();
-  fluid_mutex_lock(synth->busy); /* Here comes the audio thread. Lock the synth. */
+
+/*   fluid_mutex_lock(synth->busy); /\* Here comes the audio thread. Lock the synth. *\/ */
   
   fluid_check_fpe("??? Just starting up ???");  
 
@@ -1661,15 +1740,17 @@ fluid_synth_one_block(fluid_synth_t* synth, int do_not_mix_fx_to_out)
     if (_PLAYING(voice)) {
       double prof_ref_voice = fluid_profile_ref();
 
-      /* The output associated with a MIDI channel is wrapped around using the number of 
-       * audio groups as modulo divider.
-       * This is typically the number of output channels on the 'sound card', as long as the
-       * LADSPA Fx unit is not used. In case of LADSPA unit, think of it as subgroups on a mixer.
+      /* The output associated with a MIDI channel is wrapped around
+       * using the number of audio groups as modulo divider.  This is
+       * typically the number of output channels on the 'sound card',
+       * as long as the LADSPA Fx unit is not used. In case of LADSPA
+       * unit, think of it as subgroups on a mixer.
+       *
        * For example: Assume that the number of groups is set to 2.
-       * Then MIDI channel 1, 3, 5, 7 etc. go to output 1, channels 2, 4, 6, 8 etc to output 2. 
-       * Or assume 3 groups:
-       * Then MIDI channels 1, 4, 7, 10 etc go to output 1;  2, 5, 8, 11 etc to output 2,
-       * 3, 6, 9, 12 etc to output 3.
+       * Then MIDI channel 1, 3, 5, 7 etc. go to output 1, channels 2,
+       * 4, 6, 8 etc to output 2.  Or assume 3 groups: Then MIDI
+       * channels 1, 4, 7, 10 etc go to output 1; 2, 5, 8, 11 etc to
+       * output 2, 3, 6, 9, 12 etc to output 3.
        */
       auchan = fluid_channel_get_num(fluid_voice_get_channel(voice));
       auchan %= synth->audio_groups;
@@ -1741,7 +1822,8 @@ fluid_synth_one_block(fluid_synth_t* synth, int do_not_mix_fx_to_out)
   {float num=1;while (num != 0){num*=0.5;};};
 #endif
   fluid_check_fpe("??? Remainder of synth_one_block ???");
-  fluid_mutex_unlock(synth->busy); /* Allow other threads to touch the synth */
+
+/*   fluid_mutex_unlock(synth->busy); /\* Allow other threads to touch the synth *\/ */
 
   return 0;  
 }
@@ -1754,28 +1836,29 @@ fluid_synth_one_block(fluid_synth_t* synth, int do_not_mix_fx_to_out)
  * of the algorithm previously in fluid_synth_alloc_voice.
  */
 fluid_voice_t*
-fluid_synth_free_voice_by_kill (fluid_synth_t* synth)
+fluid_synth_free_voice_by_kill(fluid_synth_t* synth)
 {
   int i;
-  fluid_real_t best_prio=999999.;
+  fluid_real_t best_prio = 999999.;
   fluid_real_t this_voice_prio;
   fluid_voice_t* voice;
   int best_voice_index=-1;
 
-  fluid_mutex_lock(synth->busy); /* Don't interfere with the audio thread */
-  fluid_mutex_unlock(synth->busy);
+/*   fluid_mutex_lock(synth->busy); /\* Don't interfere with the audio thread *\/ */
+/*   fluid_mutex_unlock(synth->busy); */
 
   for (i = 0; i < synth->nvoice; i++) {
     
     voice = synth->voice[i];
     
     /* safeguard against an available voice. */
-    if (_AVAILABLE(voice))
+    if (_AVAILABLE(voice)) {
       return voice;
-    
+    }
+
     /* Determine, how 'important' a voice is.
      * Start with an arbitrary number */
-    this_voice_prio=10000.; 
+    this_voice_prio = 10000.; 
     
     /* Is this voice on the drum channel? 
      * Then it is very important.
@@ -1784,13 +1867,15 @@ fluid_synth_free_voice_by_kill (fluid_synth_t* synth)
      * of the time in release phase.
      */
     if (voice->chan == 9){
-      this_voice_prio+=4000;
+      this_voice_prio += 4000;
+
     } else if (_RELEASED(voice)){
       /* The key for this voice has been released. Consider it much less important
        * than a voice, which is still held.
        */
-      this_voice_prio-=2000.;
-    };
+      this_voice_prio -= 2000.;
+    }
+
     if (_SUSTAINED(voice)){
       /* The sustain pedal is held down on this channel.
        * Consider it less important than non-sustained channels.
@@ -1798,8 +1883,8 @@ fluid_synth_free_voice_by_kill (fluid_synth_t* synth)
        * is used to play 'more-voices-than-fingers', so it shouldn't hurt
        * if we kill one voice.
        */
-      this_voice_prio-=1000;
-    };
+      this_voice_prio -= 1000;
+    }
         
     /* We are not enthusiastic about releasing voices, which have just been started.
      * Otherwise hitting a chord may result in killing notes belonging to that very same
@@ -1807,12 +1892,12 @@ fluid_synth_free_voice_by_kill (fluid_synth_t* synth)
      * So subtract the age of the voice from the priority - an older voice is just a little
      * bit less important than a younger voice. 
      * This is a number between roughly 0 and 100.*/
-    this_voice_prio -= (synth->noteid-fluid_voice_get_id(voice));
+    this_voice_prio -= (synth->noteid - fluid_voice_get_id(voice));
     
     /* take a rough estimate of loudness into account. Louder voices are more important. */
     if (voice->volenv_section != FLUID_VOICE_ENVATTACK){
-      this_voice_prio += voice->volenv_val*1000.;
-    };
+      this_voice_prio += voice->volenv_val * 1000.;
+    }
 
     /* check if this voice has less priority than the previous candidate. */
     if (this_voice_prio < best_prio)
@@ -1820,11 +1905,13 @@ fluid_synth_free_voice_by_kill (fluid_synth_t* synth)
       best_prio = this_voice_prio;
   }
 
-  if (best_voice_index < 0)
+  if (best_voice_index < 0) {
     return NULL;
-  
+  }
+
   voice = synth->voice[best_voice_index];
-  fluid_voice_off (voice);
+  fluid_voice_off(voice);
+
   return voice;
 }
 
@@ -1836,9 +1923,10 @@ fluid_synth_alloc_voice(fluid_synth_t* synth, fluid_sample_t* sample, int chan,
 {
   int i, k;
   fluid_voice_t* voice = NULL;
+  fluid_channel_t* channel = NULL;
 
-  fluid_mutex_lock(synth->busy); /* Don't interfere with the audio thread */
-  fluid_mutex_unlock(synth->busy);
+/*   fluid_mutex_lock(synth->busy); /\* Don't interfere with the audio thread *\/ */
+/*   fluid_mutex_unlock(synth->busy); */
 
   /* If there is another voice process on the same channel and key,
      advance it to the release phase. */
@@ -1854,7 +1942,7 @@ fluid_synth_alloc_voice(fluid_synth_t* synth, fluid_sample_t* sample, int chan,
 
   /* No success yet? Then stop a running voice. */
   if (voice == NULL) {
-    voice = fluid_synth_free_voice_by_kill (synth);
+    voice = fluid_synth_free_voice_by_kill(synth);
   }
 
   if (voice == NULL) {
@@ -1871,15 +1959,19 @@ fluid_synth_alloc_voice(fluid_synth_t* synth, fluid_sample_t* sample, int chan,
     }
 
     FLUID_LOG(FLUID_INFO, "noteon\t%d\t%d\t%d\t%05d\t%.3f\t%.3f\t%.3f\t%d", 
-	     chan, key, vel, synth->noteid, 
+	     chan, key, vel, synth->storeid, 
 	     (float) synth->ticks / 44100.0f, 
 	     (fluid_curtime() - synth->start) / 1000.0f,
 	     0.0f,
 	     k);
   }
 
-  if (fluid_voice_init(voice, sample, synth->channel[chan], key, vel, 
-		      synth->noteid, synth->ticks, synth->gain) != FLUID_OK) {
+  if (chan >= 0) {
+	  channel = synth->channel[chan];
+  }
+
+  if (fluid_voice_init(voice, sample, channel, key, vel, 
+		       synth->storeid, synth->ticks, synth->gain) != FLUID_OK) {
     FLUID_LOG(FLUID_WARN, "Failed to initialize voice");
     return NULL;
   }
@@ -1966,8 +2058,8 @@ void fluid_synth_kill_by_exclusive_class(fluid_synth_t* synth, fluid_voice_t* ne
  */
 void fluid_synth_start_voice(fluid_synth_t* synth, fluid_voice_t* voice)
 {
-  fluid_mutex_lock(synth->busy); /* Don't interfere with the audio thread */
-  fluid_mutex_unlock(synth->busy);
+/*   fluid_mutex_lock(synth->busy); /\* Don't interfere with the audio thread *\/ */
+/*   fluid_mutex_unlock(synth->busy); */
 
   /* Find the exclusive class of this voice. If set, kill all voices
    * that match the exclusive class and are younger than the first
@@ -2202,6 +2294,24 @@ fluid_sfont_t* fluid_synth_get_sfont_by_id(fluid_synth_t* synth, unsigned int id
   return NULL;  
 }
 
+/* fluid_synth_get_sfont_by_name
+ *
+ */
+fluid_sfont_t* fluid_synth_get_sfont_by_name(fluid_synth_t* synth, char *name)
+{
+  fluid_list_t* list = synth->sfont;
+  fluid_sfont_t* sfont;
+
+  while (list) {
+    sfont = (fluid_sfont_t*) fluid_list_get(list);
+    if (FLUID_STRCMP(fluid_sfont_get_name(sfont), name) == 0) {
+      return sfont;
+    }
+    list = fluid_list_next(list);
+  }
+  return NULL;  
+}
+
 /*
  * fluid_synth_get_channel_preset
  */
@@ -2315,8 +2425,9 @@ double fluid_synth_get_reverb_width(fluid_synth_t* synth)
 void fluid_synth_release_voice_on_same_note(fluid_synth_t* synth, int chan, int key){
   int i;
   fluid_voice_t* voice;
-  fluid_mutex_lock(synth->busy); /* Don't interfere with the audio thread */
-  fluid_mutex_unlock(synth->busy);
+
+/*   fluid_mutex_lock(synth->busy); /\* Don't interfere with the audio thread *\/ */
+/*   fluid_mutex_unlock(synth->busy); */
 
   for (i = 0; i < synth->nvoice; i++) {
     voice = synth->voice[i];
@@ -2639,12 +2750,67 @@ fluid_synth_set_gen(fluid_synth_t* synth, int chan, int param, float value)
     return FLUID_FAILED;     
   }
 
-  fluid_channel_set_gen(synth->channel[chan], param, value);
+  fluid_channel_set_gen(synth->channel[chan], param, value, 0);
 
   for (i = 0; i < synth->nvoice; i++) {
     voice = synth->voice[i];
     if (voice->chan == chan) {
-      fluid_voice_set_param(voice, param, value);
+      fluid_voice_set_param(voice, param, value, 0);
+    }
+  }
+
+  return FLUID_OK;  
+}
+
+/** Change the value of a generator. This function allows to control
+    all synthesis parameters in real-time. The changes are additive,
+    i.e. they add up to the existing parameter value. This function is
+    similar to sending an NRPN message to the synthesizer. The
+    function accepts a float as the value of the parameter. The
+    parameter numbers and ranges are described in the SoundFont 2.01
+    specification, paragraph 8.1.3, page 48. See also
+    'fluid_gen_type'.
+ 
+    Using the fluid_synth_set_gen2() function, it is possible to set
+    the absolute value of a generator. This is an extension to the
+    SoundFont standard. If 'absolute' is non-zero, the value of the
+    generator specified in the SoundFont is completely ignored and the
+    generator is fixed to the value passed as argument. To undo this
+    behavior, you must call fluid_synth_set_gen2 again, with
+    'absolute' set to 0 (and possibly 'value' set to zero).
+ 
+    If 'normalized' is non-zero, the value is supposed to be
+    normalized between 0 and 1. Before applying the value, it will be
+    scaled and shifted to the range defined in the SoundFont
+    specifications.
+
+ */
+int 
+fluid_synth_set_gen2(fluid_synth_t* synth, int chan, int param, 
+		     float value, int absolute, int normalized)
+{
+  int i;
+  fluid_voice_t* voice;
+  float v;
+
+  if ((chan < 0) || (chan >= synth->midi_channels)) {
+    FLUID_LOG(FLUID_WARN, "Channel out of range");
+    return FLUID_FAILED;     
+  }
+
+  if ((param < 0) || (param >= GEN_LAST)) {
+    FLUID_LOG(FLUID_WARN, "Parameter number out of range");
+    return FLUID_FAILED;     
+  }
+
+  v = (normalized)? fluid_gen_scale(param, value) : value;
+
+  fluid_channel_set_gen(synth->channel[chan], param, v, absolute);
+
+  for (i = 0; i < synth->nvoice; i++) {
+    voice = synth->voice[i];
+    if (voice->chan == chan) {
+      fluid_voice_set_param(voice, param, v, absolute);
     }
   }
 
@@ -2709,3 +2875,55 @@ int fluid_synth_handle_midi_event(void* data, fluid_midi_event_t* event)
   return FLUID_FAILED;
 }
 
+
+int fluid_synth_start(fluid_synth_t* synth, unsigned int id, fluid_preset_t* preset, 
+		      int audio_chan, int midi_chan, int key, int vel)
+{
+  int r;
+
+  /* check the ranges of the arguments */
+  if ((midi_chan < 0) || (midi_chan >= synth->midi_channels)) {
+    FLUID_LOG(FLUID_WARN, "Channel out of range");
+    return FLUID_FAILED;
+  }
+
+  if ((key < 0) || (key >= 128)) {
+    FLUID_LOG(FLUID_WARN, "Key out of range");
+    return FLUID_FAILED;
+  }
+
+  if ((vel <= 0) || (vel >= 128)) {
+    FLUID_LOG(FLUID_WARN, "Velocity out of range");
+    return FLUID_FAILED;
+  }
+
+  fluid_mutex_lock(synth->busy); /* One at a time, please */
+
+  synth->storeid = id;
+  r = fluid_preset_noteon(preset, synth, midi_chan, key, vel);
+
+  fluid_mutex_unlock(synth->busy);
+
+  return r;
+}
+
+int fluid_synth_stop(fluid_synth_t* synth, unsigned int id)
+{
+  int i;
+  fluid_voice_t* voice;
+  int status = FLUID_FAILED;
+  int count = 0;
+  
+  for (i = 0; i < synth->nvoice; i++) {
+
+    voice = synth->voice[i];
+
+    if (_ON(voice) && (fluid_voice_get_id(voice) == id)) {
+	    count++;
+      fluid_voice_noteoff(voice);
+      status = FLUID_OK;
+    }
+  }
+
+  return status;
+}
