@@ -71,7 +71,7 @@ void fluid_voice_config()
 
 #ifdef ENABLE_SSE
   sse_t* sse_a;
-  interp_coeff_sse=(sse_t*)FLUID_ALIGN16BYTE(&interp_coeff_sse_mem);
+  interp_coeff_sse = (sse_t*)FLUID_ALIGN16BYTE(&interp_coeff_sse_mem);
 #endif
 
   /* Initialize the coefficients for the interpolation. The math comes
@@ -133,7 +133,7 @@ void fluid_voice_config()
  * new_fluid_voice
  */ 
 fluid_voice_t* 
-new_fluid_voice(fluid_real_t sample_rate)
+new_fluid_voice(fluid_real_t output_rate)
 {
   fluid_voice_t* voice;
   voice = FLUID_NEW(fluid_voice_t);
@@ -147,7 +147,7 @@ new_fluid_voice(fluid_real_t sample_rate)
   voice->vel = 0;
   voice->channel = NULL;
   voice->sample = NULL;
-  voice->sample_rate = sample_rate;
+  voice->output_rate = output_rate;
 
   /* The 'sustain' and 'finished' segments of the volume / modulation
    * envelope are constant. They are never affected by any modulator 
@@ -200,8 +200,8 @@ delete_fluid_voice(fluid_voice_t* voice)
  */
 int 
 fluid_voice_init(fluid_voice_t* voice, fluid_sample_t* sample, 
-		fluid_channel_t* channel, int key, int vel, 
-		unsigned int id, unsigned int start,fluid_real_t gain)
+		 fluid_channel_t* channel, int key, int vel, unsigned int id, 
+		 unsigned int start_time, fluid_real_t gain)
 {
   /* Note: The voice parameters will be initialized later, when the
    * generators have been retrieved from the sound font. Here, only
@@ -213,10 +213,9 @@ fluid_voice_init(fluid_voice_t* voice, fluid_sample_t* sample,
   voice->key = (unsigned char) key;
   voice->vel = (unsigned char) vel;
   voice->channel = channel;
-  voice->preset = fluid_channel_get_preset(channel);
   voice->mod_count = 0;
   voice->sample = sample;
-  voice->start = start;
+  voice->start_time = start_time;
   voice->ticks = 0;
   voice->debug = 0;
   voice->has_looped = 0; /* Will be set during voice_write when the 2nd loop point is reached */
@@ -295,6 +294,18 @@ float fluid_voice_gen_get(fluid_voice_t* voice, int gen)
   return voice->gen[gen].val;
 }
 
+fluid_real_t fluid_voice_gen_value(fluid_voice_t* voice, int num)
+{
+	/* This is an extension to the SoundFont standard. More
+	 * documentation is available at the fluid_synth_set_gen2()
+	 * function. */
+	if (voice->gen[num].flags == GEN_ABS_NRPN) {
+		return (fluid_real_t) voice->gen[num].nrpn;
+	} else {
+		return (fluid_real_t) (voice->gen[num].val + voice->gen[num].mod + voice->gen[num].nrpn);
+	}
+}
+
 
 /*
  * fluid_voice_write
@@ -511,7 +522,7 @@ fluid_voice_write(fluid_voice_t* voice,
      * the sample loop
      */
 #if 0
-    printf("%i %i %i %i %i %i\n", voice->has_looped, voice->sample->amplitude_that_reaches_noise_floor_is_valid, voice->loop_start, voice->loop_end, voice->sample->loopstart, voice->sample->loopend  );
+    printf("%i %i %i %i %i %i\n", voice->has_looped, voice->sample->amplitude_that_reaches_noise_floor_is_valid, voice->loopstart, voice->loopend, voice->sample->loopstart, voice->sample->loopend  );
 #endif
     
     /* Is the playing pointer already in the loop? */
@@ -583,40 +594,74 @@ fluid_voice_write(fluid_voice_t* voice,
   /* Convert the 'speed' through the original waveform to a
    * representation 'integer part' and 'fractional part' */
   fluid_phase_set_float(dsp_phase_incr, incr);
-  incr *= 1.00000000000001; /* gcc optimization problem. Quick fix. */
+  /* incr *= 1.00000000000001;  FIXME: gcc optimization problem. Quick fix. [Commented out. Don't understand the problem, PH]  */
   fluid_check_fpe("voice_write phase calculation");
 
   /* Check, if we are really making progress through the original
    * sample.  If the step size is rounded to 0, the DSP loop would get
    * stuck. */
-  if (fluid_phase_index(dsp_phase) == 0 && fluid_phase_fract(dsp_phase) == 0){
-      fluid_phase_fract(dsp_phase)=1;
+  /* [PH] I commented this out. The first time a voice is called, dsp_phase
+   * will be zero. Settings the fractionnal part of the phase to
+   * non-zero, causes the sample to be interpolated even if it is
+   * played a the root key (cfr. fluid_dsp_float.c:96: 
+   *  if ((fluid_phase_fract(dsp_phase) == 0))) 
+   *
+   * I replaced it with a check on the phase increment.
+   */
+/*   if ((fluid_phase_index(dsp_phase) == 0) && (fluid_phase_fract(dsp_phase) == 0)) { */
+/*       fluid_phase_fract(dsp_phase) = 1; */
+/*   } */
+  if ((fluid_phase_index(dsp_phase_incr) == 0) && (fluid_phase_fract(dsp_phase_incr) == 0)) {
+      fluid_phase_fract(dsp_phase_incr) = 1;
   }
 
   /*************** resonant filter ******************/
 
   /* calculate the frequency of the resonant filter in Hz */
   fres = fluid_ct2hz(voice->fres 
-		    + voice->modlfo_val * voice->modlfo_to_fc 
-		    + voice->modenv_val * voice->modenv_to_fc);
+		     + voice->modlfo_val * voice->modlfo_to_fc 
+		     + voice->modenv_val * voice->modenv_to_fc);
 
   /* Testcase for filter resonance: Use the following line, and press
    * A3. There should be a _very_ pronounced resonant peak, which
    * drops down, when moving the pitch bend wheel only slightly. */
   //fres=440;voice->q_lin=100;
 
-  if (fres < 5) {
-      fres = 5;
-  } else if ((fres > FLUID_MAX_AUDIBLE_FILTER_FC)
-	     && (voice->q_lin < FLUID_MIN_AUDIBLE_FILTER_Q)) {
-    /* Beyond a certain point the filter does not change the sound
-     * much.  Turn it off to save CPU cycles. */
-    /* Fixme: When the filter has been on once for a voice, keep it on forever. */
-    dsp_use_filter_flag = 0;
+
+  /* FIXME - Still potential for a click during turn on, can we interpolate
+     between 20khz cutoff and 0 Q? */
+
+  /* if filter has not yet started and filter cutoff and Q don't
+     exceed "audible" thresholds, then don't turn on the filter.
+     Once the filter is turned on, it remains on. */
+/*   if (voice->filter_startup */
+/*       && (fres > FLUID_MAX_AUDIBLE_FILTER_FC) */
+/*       && (voice->q_lin < FLUID_MIN_AUDIBLE_FILTER_Q)) { */
+/*     dsp_use_filter_flag = 0; */
+/*   } else if (fres < 5) { */
+/*     fres = 5; */
+/*   } */
+
+  /* I removed the optimization of turning the filter off when the
+   * resonance frequence is above the maximum frequency. Instead, the
+   * filter frequence is set to a maximum of 0.45 times the sampling
+   * rate. For a 44100 kHz sampling rate, this amounts to 19845
+   * Hz. The reasing is that were problems with anti-aliasing when the
+   * synthesizer was run at lower sampling rates. Thanks to Stephan
+   * Tassart for pointing me to this bug. By turning the filter on and
+   * clipping the maximum filter frequency at 0.45*srate, the filter
+   * is used as an anti-aliasing filter. */
+
+  if (fres > 0.45f * voice->output_rate) {
+    fres = 0.45f * voice->output_rate;	  
+  } else if (fres < 5) {
+    fres = 5;
   }
-   
-   if (dsp_use_filter_flag && (abs(fres - voice->last_fres) > 0.01)) {
-     
+
+
+  /* if filter enabled and there is a significant frequency change.. */
+  if (/*dsp_use_filter_flag &&*/ (abs(fres - voice->last_fres) > 0.01)) {
+
      /* The filter coefficients have to be recalculated (filter
       * parameters have changed). Recalculation for various reasons is
       * forced by setting last_fres to -1.  The flag filter_startup
@@ -653,7 +698,7 @@ fluid_voice_write(fluid_voice_t* voice,
      /* both b0 -and- b2 */
      fluid_real_t b02_temp = b1_temp * 0.5f;
 
-     if (voice->filter_startup){
+     if (voice->filter_startup) {
        /* The filter is calculated, because the voice was started up.
 	* In this case set the filter coefficients without delay.
 	*/
@@ -718,13 +763,13 @@ fluid_voice_write(fluid_voice_t* voice,
    * may require several runs. */
   fluid_check_fpe("voice_write DSP processing");
 
-  if (((_SAMPLEMODE(voice) == FLUID_LOOP) && (voice->volenv_section < FLUID_VOICE_ENVRELEASE)) 
+  if (((_SAMPLEMODE(voice) == FLUID_LOOP_UNTIL_RELEASE) && (voice->volenv_section < FLUID_VOICE_ENVRELEASE)) 
       || (_SAMPLEMODE(voice) == FLUID_LOOP_DURING_RELEASE)) {
 
     /* At which index does the loop point occur in the output buffer?
      * This calculates the first index in the buffer, which uses
      * sample data taken after the looparound. */
-    end_in_buffer = fluid_phase_steps(dsp_phase, voice->loop_end, incr);
+    end_in_buffer = fluid_phase_steps(dsp_phase, voice->loopend, incr);
     
     if (end_in_buffer >= FLUID_BUFSIZE) {
       /* The loop occurs after the end of the buffer. 
@@ -750,9 +795,9 @@ fluid_voice_write(fluid_voice_t* voice,
 #include "fluid_dsp_core.c"
       
 	/* loop */
-	fluid_phase_sub_int(dsp_phase, voice->loop_end - voice->loop_start); 
+	fluid_phase_sub_int(dsp_phase, voice->loopend - voice->loopstart); 
 	start = end_in_buffer;
-	end_in_buffer += fluid_phase_steps(dsp_phase, voice->loop_end, incr);
+	end_in_buffer += fluid_phase_steps(dsp_phase, voice->loopend, incr);
       }
       voice->has_looped=1;
       dsp_start = start;
@@ -764,7 +809,7 @@ fluid_voice_write(fluid_voice_t* voice,
     /* Not looping right now. */
     
     dsp_start = 0;
-    end_in_buffer = fluid_phase_steps(dsp_phase, voice->sample_end, incr);
+    end_in_buffer = fluid_phase_steps(dsp_phase, voice->end, incr);
 
     if (end_in_buffer >= FLUID_BUFSIZE) {
       /* Run the whole buffer at once */
@@ -1030,7 +1075,7 @@ int calculate_hold_decay_buffers(fluid_voice_t* voice, int gen_base,
   /* Each DSP loop processes FLUID_BUFSIZE samples. */
  
   /* round to next full number of buffers */
-  buffers = (int)(((fluid_real_t)voice->sample_rate * seconds) 
+  buffers = (int)(((fluid_real_t)voice->output_rate * seconds) 
 		  / (fluid_real_t)FLUID_BUFSIZE
 		  +0.5);
 
@@ -1118,23 +1163,26 @@ fluid_voice_update_param(fluid_voice_t* voice, int gen)
     }
     voice->root_pitch = fluid_ct2hz(voice->root_pitch);
     if (voice->sample != NULL) {
-      voice->root_pitch *= (fluid_real_t) voice->sample_rate / voice->sample->samplerate;
+      voice->root_pitch *= (fluid_real_t) voice->output_rate / voice->sample->samplerate;
     }
     break;
 
   case GEN_FILTERFC:
-    /* The resonance frequency is converted from absolute cents to midicents
-     * .val and .mod are both used, this permits real-time modulation.
-     * The allowed range is tested in the 'fluid_ct2hz' function [PH,20021214] 
+    /* The resonance frequency is converted from absolute cents to
+     * midicents .val and .mod are both used, this permits real-time
+     * modulation.  The allowed range is tested in the 'fluid_ct2hz'
+     * function [PH,20021214]
      */
     voice->fres = _GEN(voice, GEN_FILTERFC);
 
-    /* The synthesis loop will have to recalculate the filter coefficients. */
+    /* The synthesis loop will have to recalculate the filter
+     * coefficients. */
     voice->last_fres = -1.0f;
     break;
 
   case GEN_FILTERQ:
-    /* The generator contains 'centibels' (1/10 dB) => divide by 10 to obtain dB */
+    /* The generator contains 'centibels' (1/10 dB) => divide by 10 to
+     * obtain dB */
     q_dB = _GEN(voice, GEN_FILTERQ) / 10.0f;
 	    
     /* Range: SF2.01 section 8.1.3 # 8 (convert from cB to dB => /10) */
@@ -1196,7 +1244,7 @@ fluid_voice_update_param(fluid_voice_t* voice, int gen)
   case GEN_MODLFODELAY:
     x = _GEN(voice, GEN_MODLFODELAY);
     fluid_clip(x, -12000.0f, 5000.0f);
-    voice->modlfo_delay = (unsigned int) (voice->sample_rate * fluid_tc2sec_delay(x));
+    voice->modlfo_delay = (unsigned int) (voice->output_rate * fluid_tc2sec_delay(x));
     break;
 
   case GEN_MODLFOFREQ:
@@ -1205,7 +1253,7 @@ fluid_voice_update_param(fluid_voice_t* voice, int gen)
      */
     x = _GEN(voice, GEN_MODLFOFREQ);
     fluid_clip(x, -16000.0f, 4500.0f);
-    voice->modlfo_incr = (4.0f * FLUID_BUFSIZE * fluid_act2hz(x) / voice->sample_rate);
+    voice->modlfo_incr = (4.0f * FLUID_BUFSIZE * fluid_act2hz(x) / voice->output_rate);
     break;
 
   case GEN_VIBLFOFREQ:
@@ -1216,13 +1264,13 @@ fluid_voice_update_param(fluid_voice_t* voice, int gen)
      */
     x = _GEN(voice, GEN_VIBLFOFREQ);
     fluid_clip(x, -16000.0f, 4500.0f);
-    voice->viblfo_incr = (4.0f * FLUID_BUFSIZE * fluid_act2hz(x) / voice->sample_rate);
+    voice->viblfo_incr = (4.0f * FLUID_BUFSIZE * fluid_act2hz(x) / voice->output_rate);
     break;
 		
   case GEN_VIBLFODELAY:
     x = _GEN(voice,GEN_VIBLFODELAY);
     fluid_clip(x, -12000.0f, 5000.0f);
-    voice->viblfo_delay = (unsigned int) (voice->sample_rate * fluid_tc2sec_delay(x));
+    voice->viblfo_delay = (unsigned int) (voice->output_rate * fluid_tc2sec_delay(x));
     break;
 			
   case GEN_VIBLFOTOPITCH:
@@ -1289,7 +1337,7 @@ fluid_voice_update_param(fluid_voice_t* voice, int gen)
   case GEN_STARTADDROFS:              /* SF2.01 section 8.1.3 # 0 */
   case GEN_STARTADDRCOARSEOFS:        /* SF2.01 section 8.1.3 # 4 */
     if (voice->sample != NULL) {
-      voice->sample_start = (voice->sample->start
+      voice->start = (voice->sample->start
 			     + (int) _GEN(voice, GEN_STARTADDROFS)
 			     + 32768 * (int) _GEN(voice, GEN_STARTADDRCOARSEOFS));
       voice->check_sample_sanity_flag = FLUID_SAMPLESANITY_CHECK;
@@ -1298,7 +1346,7 @@ fluid_voice_update_param(fluid_voice_t* voice, int gen)
   case GEN_ENDADDROFS:                 /* SF2.01 section 8.1.3 # 1 */
   case GEN_ENDADDRCOARSEOFS:           /* SF2.01 section 8.1.3 # 12 */
     if (voice->sample != NULL) { 
-      voice->sample_end = (voice->sample->end
+      voice->end = (voice->sample->end
 			   + (int) _GEN(voice, GEN_ENDADDROFS)
 			   + 32768 * (int) _GEN(voice, GEN_ENDADDRCOARSEOFS));
       voice->check_sample_sanity_flag = FLUID_SAMPLESANITY_CHECK;
@@ -1307,7 +1355,7 @@ fluid_voice_update_param(fluid_voice_t* voice, int gen)
   case GEN_STARTLOOPADDROFS:           /* SF2.01 section 8.1.3 # 2 */
   case GEN_STARTLOOPADDRCOARSEOFS:     /* SF2.01 section 8.1.3 # 45 */
     if (voice->sample != NULL) {
-      voice->loop_start = (voice->sample->loopstart
+      voice->loopstart = (voice->sample->loopstart
 				  + (int) _GEN(voice, GEN_STARTLOOPADDROFS)
 				  + 32768 * (int) _GEN(voice, GEN_STARTLOOPADDRCOARSEOFS));
       voice->check_sample_sanity_flag = FLUID_SAMPLESANITY_CHECK;
@@ -1317,7 +1365,7 @@ fluid_voice_update_param(fluid_voice_t* voice, int gen)
   case GEN_ENDLOOPADDROFS:             /* SF2.01 section 8.1.3 # 3 */
   case GEN_ENDLOOPADDRCOARSEOFS:       /* SF2.01 section 8.1.3 # 50 */
     if (voice->sample != NULL) {
-      voice->loop_end = (voice->sample->loopend
+      voice->loopend = (voice->sample->loopend
 				+ (int) _GEN(voice, GEN_ENDLOOPADDROFS)
 				+ 32768 * (int) _GEN(voice, GEN_ENDLOOPADDRCOARSEOFS));
       voice->check_sample_sanity_flag = FLUID_SAMPLESANITY_CHECK;
@@ -1325,9 +1373,9 @@ fluid_voice_update_param(fluid_voice_t* voice, int gen)
     break;
 
     /* Conversion functions differ in range limit */
-#define NUM_BUFFERS_DELAY(_v)   (unsigned int) (voice->sample_rate * fluid_tc2sec_delay(_v) / FLUID_BUFSIZE)
-#define NUM_BUFFERS_ATTACK(_v)  (unsigned int) (voice->sample_rate * fluid_tc2sec_attack(_v) / FLUID_BUFSIZE)
-#define NUM_BUFFERS_RELEASE(_v) (unsigned int) (voice->sample_rate * fluid_tc2sec_release(_v) / FLUID_BUFSIZE)
+#define NUM_BUFFERS_DELAY(_v)   (unsigned int) (voice->output_rate * fluid_tc2sec_delay(_v) / FLUID_BUFSIZE)
+#define NUM_BUFFERS_ATTACK(_v)  (unsigned int) (voice->output_rate * fluid_tc2sec_attack(_v) / FLUID_BUFSIZE)
+#define NUM_BUFFERS_RELEASE(_v) (unsigned int) (voice->output_rate * fluid_tc2sec_release(_v) / FLUID_BUFSIZE)
 
     /* volume envelope
      *
@@ -1338,7 +1386,8 @@ fluid_voice_update_param(fluid_voice_t* voice, int gen)
   case GEN_VOLENVDELAY:                /* SF2.01 section 8.1.3 # 33 */
     x = _GEN(voice, GEN_VOLENVDELAY);
     fluid_clip(x, -12000.0f, 5000.0f);
-    voice->volenv_data[FLUID_VOICE_ENVDELAY].count = NUM_BUFFERS_DELAY(x);
+    count = NUM_BUFFERS_DELAY(x);
+    voice->volenv_data[FLUID_VOICE_ENVDELAY].count = count;
     voice->volenv_data[FLUID_VOICE_ENVDELAY].coeff = 0.0f;
     voice->volenv_data[FLUID_VOICE_ENVDELAY].incr = 0.0f;
     voice->volenv_data[FLUID_VOICE_ENVDELAY].min = -1.0f;
@@ -1369,12 +1418,13 @@ fluid_voice_update_param(fluid_voice_t* voice, int gen)
   case GEN_VOLENVDECAY:               /* SF2.01 section 8.1.3 # 36 */
   case GEN_VOLENVSUSTAIN:             /* SF2.01 section 8.1.3 # 37 */
   case GEN_KEYTOVOLENVDECAY:          /* SF2.01 section 8.1.3 # 40 */
+    y = 1.0f - 0.001f * _GEN(voice, GEN_VOLENVSUSTAIN);
+    fluid_clip(y, 0.0f, 1.0f);
     count = calculate_hold_decay_buffers(voice, GEN_VOLENVDECAY, GEN_KEYTOVOLENVDECAY, 1); /* 1 for decay */
     voice->volenv_data[FLUID_VOICE_ENVDECAY].count = count;
     voice->volenv_data[FLUID_VOICE_ENVDECAY].coeff = 1.0f;
     voice->volenv_data[FLUID_VOICE_ENVDECAY].incr = count ? -1.0f / count : 0.0f;
-    /* fluid_cb2amp checks the range */
-    voice->volenv_data[FLUID_VOICE_ENVDECAY].min = fluid_cb2amp(_GEN(voice, GEN_VOLENVSUSTAIN)); 
+    voice->volenv_data[FLUID_VOICE_ENVDECAY].min = y;
     voice->volenv_data[FLUID_VOICE_ENVDECAY].max = 2.0f;
     break;
 
@@ -1583,7 +1633,7 @@ fluid_voice_noteoff(fluid_voice_t* voice)
         fluid_real_t env_value = - ((-200. * log (amp) / log (10.) - attn_and_lfo) / 960.0 - 1);
         env_value = (env_value < 0.0 ? 0.0 : env_value);
         env_value = (env_value > 1.0 ? 1.0 : env_value);
-        voice->volenv_val=env_value;
+        voice->volenv_val = env_value;
       }
     }
     voice->volenv_section = FLUID_VOICE_ENVRELEASE;
@@ -1765,8 +1815,8 @@ fluid_real_t fluid_voice_get_lower_boundary_for_attenuation(fluid_voice_t* voice
     if ((mod->dest == GEN_ATTENUATION)  
 	&& ((mod->flags1 & FLUID_MOD_CC) || (mod->flags2 & FLUID_MOD_CC))) {
       
-      fluid_real_t current_val=fluid_mod_get_value(mod, voice->channel, voice);
-      fluid_real_t v=fabs(mod->amount);
+      fluid_real_t current_val = fluid_mod_get_value(mod, voice->channel, voice);
+      fluid_real_t v = fabs(mod->amount);
 
       if ((mod->src1 == FLUID_MOD_PITCHWHEEL) 
 	  || (mod->flags1 & FLUID_MOD_BIPOLAR) 
@@ -1822,71 +1872,71 @@ void fluid_voice_check_sample_sanity(fluid_voice_t* voice)
 #if 0
     printf("Sample from %i to %i\n",voice->sample->start, voice->sample->end);
     printf("Sample loop from %i %i\n",voice->sample->loopstart, voice->sample->loopend);
-    printf("Playback from %i to %i\n", voice->sample_start, voice->sample_end);
-    printf("Playback loop from %i to %i\n",voice->loop_start, voice->loop_end);
+    printf("Playback from %i to %i\n", voice->start, voice->end);
+    printf("Playback loop from %i to %i\n",voice->loopstart, voice->loopend);
 #endif
 
     /* Keep the start point within the sample data */
-    if (voice->sample_start < min_index_nonloop){
-	voice->sample_start = min_index_nonloop;
-    } else if (voice->sample_start > max_index_nonloop){
-	voice->sample_start = max_index_nonloop;
+    if (voice->start < min_index_nonloop){
+	voice->start = min_index_nonloop;
+    } else if (voice->start > max_index_nonloop){
+	voice->start = max_index_nonloop;
     }
 
     /* Keep the end point within the sample data */
-    if (voice->sample_end < min_index_nonloop){
-      voice->sample_end = min_index_nonloop;
-    } else if (voice->sample_end > max_index_nonloop){
-      voice->sample_end = max_index_nonloop;
+    if (voice->end < min_index_nonloop){
+      voice->end = min_index_nonloop;
+    } else if (voice->end > max_index_nonloop){
+      voice->end = max_index_nonloop;
     }
     
     /* Keep start and end point in the right order */
-    if (voice->sample_start > voice->sample_end){
-	int temp = voice->sample_start;
-	voice->sample_start = voice->sample_end;
-	voice->sample_end = temp;
+    if (voice->start > voice->end){
+	int temp = voice->start;
+	voice->start = voice->end;
+	voice->end = temp;
 	/*FLUID_LOG(FLUID_DBG, "Loop / sample sanity check: Changing order of start / end points!"); */
     }
     
     /* Zero length? */
-    if (voice->sample_start == voice->sample_end){
+    if (voice->start == voice->end){
 	fluid_voice_off(voice);
 	return;
     }
 
-    if ((_SAMPLEMODE(voice) == FLUID_LOOP) 
+    if ((_SAMPLEMODE(voice) == FLUID_LOOP_UNTIL_RELEASE) 
 	|| (_SAMPLEMODE(voice) == FLUID_LOOP_DURING_RELEASE)) {
 	/* Keep the loop start point within the sample data */
-	if (voice->loop_start < min_index_loop){
-	    voice->loop_start = min_index_loop;
-      } else if (voice->loop_start > max_index_loop){
-	voice->loop_start = max_index_loop;
+	if (voice->loopstart < min_index_loop){
+	    voice->loopstart = min_index_loop;
+      } else if (voice->loopstart > max_index_loop){
+	voice->loopstart = max_index_loop;
       }
       
       /* Keep the loop end point within the sample data */
-      if (voice->loop_end < min_index_loop){
-	voice->loop_end = min_index_loop;
-      } else if (voice->loop_end > max_index_loop){
-	voice->loop_end = max_index_loop;
+      if (voice->loopend < min_index_loop){
+	voice->loopend = min_index_loop;
+      } else if (voice->loopend > max_index_loop){
+	voice->loopend = max_index_loop;
       }
       
       /* Keep loop start and end point in the right order */
-      if (voice->loop_start > voice->loop_end){
-	int temp=voice->loop_start;
-	voice->loop_start=voice->loop_end;
-	voice->loop_end=temp;
+      if (voice->loopstart > voice->loopend){
+	int temp = voice->loopstart;
+	voice->loopstart = voice->loopend;
+	voice->loopend = temp;
 	/*FLUID_LOG(FLUID_DBG, "Loop / sample sanity check: Changing order of loop points!"); */
       }
       
       /* Loop too short? Then don't loop. */
-      if (voice->loop_end < voice->loop_start + FLUID_MIN_LOOP_SIZE){
-	  voice->gen[GEN_SAMPLEMODE].val=FLUID_UNLOOPED;      
+      if (voice->loopend < voice->loopstart + FLUID_MIN_LOOP_SIZE){
+	  voice->gen[GEN_SAMPLEMODE].val = FLUID_UNLOOPED;      
       }      
 
       /* The loop points may have changed. Obtain a new estimate for the loop volume. */
       /* Is the voice loop within the sample loop? */
-      if ((int)voice->loop_start >= (int)voice->sample->loopstart 
-	  && (int)voice->loop_end <= (int)voice->sample->loopend){
+      if ((int)voice->loopstart >= (int)voice->sample->loopstart 
+	  && (int)voice->loopend <= (int)voice->sample->loopend){
 	/* Is there a valid peak amplitude available for the loop? */
 	if (voice->sample->amplitude_that_reaches_noise_floor_is_valid){
 	  voice->amplitude_that_reaches_noise_floor_loop=voice->sample->amplitude_that_reaches_noise_floor / voice->synth_gain;
@@ -1900,22 +1950,21 @@ void fluid_voice_check_sample_sanity(fluid_voice_t* voice)
     
     /* Run startup specific code (only once, when the voice is started) */
     if (voice->check_sample_sanity_flag & FLUID_SAMPLESANITY_STARTUP){
-      if (max_index_loop-min_index_loop < FLUID_MIN_LOOP_SIZE){
-	if (_SAMPLEMODE(voice) == FLUID_LOOP 
+      if (max_index_loop - min_index_loop < FLUID_MIN_LOOP_SIZE){
+        if ((_SAMPLEMODE(voice) == FLUID_LOOP_UNTIL_RELEASE)
 	    || (_SAMPLEMODE(voice) == FLUID_LOOP_DURING_RELEASE)){
-	  voice->gen[GEN_SAMPLEMODE].val=FLUID_UNLOOPED;
+	  voice->gen[GEN_SAMPLEMODE].val = FLUID_UNLOOPED;
 	}
       }
       
       /* Set the initial phase of the voice (using the result from the
 	 start offset modulators). */
-      fluid_phase_set_int(voice->phase, voice->sample_start);
+      fluid_phase_set_int(voice->phase, voice->start);
     } /* if startup */
     
     /* Is this voice run in loop mode, or does it run straight to the
        end of the waveform data? */
-    if (((_SAMPLEMODE(voice) == FLUID_LOOP) 
-	 && (voice->volenv_section < FLUID_VOICE_ENVRELEASE)) 
+    if (((_SAMPLEMODE(voice) == FLUID_LOOP_UNTIL_RELEASE) && (voice->volenv_section < FLUID_VOICE_ENVRELEASE)) 
 	|| (_SAMPLEMODE(voice) == FLUID_LOOP_DURING_RELEASE)) {
       /* Yes, it will loop as soon as it reaches the loop point.  In
        * this case we must prevent, that the playback pointer (phase)
@@ -1929,26 +1978,27 @@ void fluid_voice_check_sample_sanity(fluid_voice_t* voice)
        * actions required.
        */
       int index_in_sample = fluid_phase_index(voice->phase);
-      if (index_in_sample >= voice->loop_end){
+      if (index_in_sample >= voice->loopend){
 	/* FLUID_LOG(FLUID_DBG, "Loop / sample sanity check: Phase after 2nd loop point!"); */
-	fluid_phase_set_int(voice->phase,voice->loop_start);
+	fluid_phase_set_int(voice->phase, voice->loopstart);
       }
     }
-/*    FLUID_LOG(FLUID_DBG, "Loop / sample sanity check: Sample from %i to %i, loop from %i to %i", voice->sample_start, voice->sample_end, voice->loop_start, voice->loop_end); */
+/*    FLUID_LOG(FLUID_DBG, "Loop / sample sanity check: Sample from %i to %i, loop from %i to %i", voice->start, voice->end, voice->loopstart, voice->loopend); */
    
     /* Sample sanity has been assured. Don't check again, until some
        sample parameter is changed by modulation. */
     voice->check_sample_sanity_flag=0;
 #if 0
-    printf("Sane? playback loop from %i to %i\n",voice->loop_start, voice->loop_end);
+    printf("Sane? playback loop from %i to %i\n", voice->loopstart, voice->loopend);
 #endif
     fluid_check_fpe("voice_check_sample_sanity");
 }
 
 
-int fluid_voice_set_param(fluid_voice_t* voice, int gen, fluid_real_t nrpn_value)
+int fluid_voice_set_param(fluid_voice_t* voice, int gen, fluid_real_t nrpn_value, int abs)
 {
   voice->gen[gen].nrpn = nrpn_value;
+  voice->gen[gen].flags = (abs)? GEN_ABS_NRPN : GEN_SET;
   fluid_voice_update_param(voice, gen);
   return FLUID_OK;
 }
