@@ -92,20 +92,6 @@ extern int optind, opterr, optopt;
 #endif
 
 
-/* is_number
- *
- * don't shoot me 
- */
-int is_number(char *a)
-{
-  for (; *a != 0; a++) {
-    if (((*a < '0') || (*a > '9')) && (*a != '-') && (*a != '+') && (*a != '.')) {
-      return 0;
-    }
-  }
-  return 1;
-}
-
 /* process_o_cmd_line_option
  *
  * Purpose:
@@ -176,18 +162,20 @@ int main(int argc, char** argv)
   char* midi_id = NULL;
   char* midi_driver = NULL;
   char* midi_device = NULL;
+  char* file = NULL;
   int audio_groups = 0;
   int audio_channels = 0;
   int with_server = 0;
   int dump = 0;
-  appname = argv[0];
+  int ladcca_connect = 1;
 #ifdef HAVE_LADCCA
   cca_args_t * cca_args;
   pthread_t cca_thread;
-  
+
   cca_args = cca_extract_args (&argc, &argv);
 #endif
 
+  appname = argv[0];
   settings = new_fluid_settings();
 
 #if !defined(WIN32) && !defined(MACINTOSH)
@@ -209,6 +197,8 @@ int main(int argc, char** argv)
       {"audio-bufsize", 1, 0, 'z'},
       {"audio-bufcount", 1, 0, 'c'},
       {"sample-rate", 1, 0, 'r'},
+      {"disable-ladcca", 1, 0, 'f'},
+      {"disable-ladcca", 0, 0, 'l'},
       {"verbose", 0, 0, 'v'},
       {"reverb", 1, 0, 'R'},
       {"chorus", 1, 0, 'C'},
@@ -222,7 +212,7 @@ int main(int argc, char** argv)
       {0, 0, 0, 0}
     };
 
-    c = getopt_long(argc, argv, "vnixdhVspm:K:L:M:a:A:s:z:c:R:C:r:G:o:g:j", 
+    c = getopt_long(argc, argv, "vnixdhVsplf:m:K:L:M:a:A:s:z:c:R:C:r:G:o:g:j", 
 		    long_options, &option_index);
     if (c == -1) {
       break;
@@ -257,6 +247,12 @@ int main(int argc, char** argv)
     case 'r':
       fluid_settings_setnum(settings, "synth.sample-rate", atof(optarg));
       break;
+    case 'f':
+      file = optarg;
+      break;
+    case 'l':			/* disable LADCCA */
+      ladcca_connect = 0;
+      break;
     case 'L':
       audio_channels = atoi(optarg);
       fluid_settings_setint(settings, "synth.audio-channels", audio_channels);
@@ -368,6 +364,14 @@ int main(int argc, char** argv)
 	  printf ("Option -r requires an argument\n");	  
 	}
 	break;
+      case 'f':
+	if (++i < argc) {
+	  file = argv[i];
+	}
+	break;
+      case 'l':			/* disable LADCCA */
+	ladcca_connect = 0;
+	break;
       case 'L':
 	if (++i < argc) {
 	  audio_channels = atoi(argv[i]);
@@ -381,7 +385,7 @@ int main(int argc, char** argv)
 	break;
       case 'K':
 	if (++i < argc) {
-	  fluid_settings_setint(settings, "synth.audio-channels", atoi(argv[i]));
+	  fluid_settings_setint(settings, "synth.midi-channels", atoi(argv[i]));
 	} else {
 	  printf ("Option -K requires an argument\n");	  
 	}
@@ -434,23 +438,29 @@ int main(int argc, char** argv)
 
   /* connect to the ladcca server */
 #ifdef HAVE_LADCCA
-  {
-    int flags;
-    char * str;
-    
-    flags = CCA_Config_Data_Set | CCA_Terminal;
+  if (ladcca_connect)
+    {
+      int flags;
+      char * str;
 
-    if (fluid_settings_str_equal(settings, "audio.driver", "jack")) {
-      flags |= CCA_Use_Jack;
+      flags = CCA_Config_Data_Set | CCA_Terminal;
+
+      /*  Removed from LADCCA? It is sufficient to just set the ALSA id or
+	  Jack client name - JG
+
+      if (fluid_settings_str_equal(settings, "audio.driver", "jack")) {
+	flags |= CCA_Use_Jack;
+      }
+      if (fluid_settings_str_equal(settings, "midi.driver", "alsa_seq")) {
+	flags |= CCA_Use_Alsa;
+      }
+      */
+
+      fluid_cca_client = cca_init (cca_args, "FluidSynth", flags, CCA_PROTOCOL (1,1));
+
+      if (fluid_cca_client)
+	fluid_settings_setint (settings, "ladcca.enable", cca_enabled (fluid_cca_client) ? 1 : 0);
     }
-    if (fluid_settings_str_equal(settings, "midi.driver", "alsa_seq")) {
-      flags |= CCA_Use_Alsa;
-    }
-    
-    fluid_cca_client = cca_init (cca_args, "FluidSynth", flags, CCA_PROTOCOL (1,1));
-    
-    fluid_settings_setint (settings, "ladcca.enable", cca_enabled (fluid_cca_client) ? 1 : 0);
-  }
 #endif /* HAVE_LADCCA */
 
 
@@ -478,7 +488,9 @@ int main(int argc, char** argv)
   }
 
   /* try to load the user or system configuration */
-  if (fluid_get_userconf(buf, 512) != NULL) {
+  if (file != NULL) {
+    fluid_source(cmd_handler, file);
+  } else if (fluid_get_userconf(buf, 512) != NULL) {
     fluid_source(cmd_handler, buf);
   } else if (fluid_get_sysconf(buf, 512) != NULL) {
     fluid_source(cmd_handler, buf);
@@ -570,7 +582,7 @@ int main(int argc, char** argv)
 
 
 #ifdef HAVE_LADCCA
-  if (cca_enabled (fluid_cca_client))
+  if (ladcca_connect && cca_enabled (fluid_cca_client))
     pthread_create (&cca_thread, NULL, cca_run, synth);
 #endif /* HAVE_LADCCA */
 
@@ -668,7 +680,6 @@ print_welcome()
 	 "under certain conditions; see the COPYING file for details.\n"
 	 "SoundFont(R) is a registered trademark of E-mu Systems, Inc.\n\n"
 	 "Type 'help' to get information on the shell commands.\n\n", FLUIDSYNTH_VERSION);
-
 }
 
 /*
@@ -694,7 +705,7 @@ print_help()
 	 "    Don't create a midi driver to read MIDI input events [default = yes]\n\n");
   printf(" -m, --midi-driver=[label]\n"
 	 "    The name of the midi driver to use [oss,alsa,alsa_seq,...]\n\n");
-  printf(" -L, --midi-channels=[num]\n"
+  printf(" -K, --midi-channels=[num]\n"
 	 "    The number of midi channels [default = 16]\n\n");
   printf(" -a, --audio-driver=[label]\n"
 	 "    The audio driver [alsa,jack,oss,dsound,...]\n\n");
@@ -714,6 +725,10 @@ print_help()
 	 "    Turn the chorus on or off [0|1|yes|no, default = on]\n\n");
   printf(" -g, --gain\n"
 	 "    Set the master gain [0 < gain < 10, default = 0.2]\n\n");
+#ifdef HAVE_LADCCA
+  printf(" -l, --disable-ladcca\n"
+	 "    Don't connect to LADCCA server");
+#endif
   printf(" -o\n"
 	 "    Define a setting, -o name=value\n\n");
   printf(" -i, --no-shell\n"
